@@ -42,7 +42,9 @@ use bincode;
 use std::{
     fs::File,
     marker::PhantomData,
+    ops::Deref,
     path::{Path, PathBuf},
+    slice::Chunks,
 };
 
 pub mod optical_sensitivities;
@@ -204,6 +206,123 @@ impl LOMBuilder {
     }
 }
 
+/// Type holding the tip-tilt values
+pub struct TipTilt(Vec<f64>);
+/// Type holding the segment tip-tilt values
+pub struct SegmentTipTilt(Vec<f64>);
+/// Type holding the segment piston values
+pub struct SegmentPiston(Vec<f64>);
+// Dereferencing
+impl Deref for TipTilt {
+    type Target = Vec<f64>;
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+impl Deref for SegmentTipTilt {
+    type Target = Vec<f64>;
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+impl Deref for SegmentPiston {
+    type Target = Vec<f64>;
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+/// Trait for the [LOM] optical metrics
+///
+/// A simple trait looking at the number of items in the [TipTilt], [SegmentTipTilt] and [SegmentPiston] metrics
+pub trait OpticalMetrics {
+    fn n_item(&self) -> usize;
+    /// Returns a [chunks] iterator with chunks the size of [n_item]
+    fn items(&self) -> Chunks<'_, f64>
+    where
+        Self: Deref<Target = Vec<f64>>,
+    {
+        self.deref().chunks(self.n_item())
+    }
+}
+impl OpticalMetrics for TipTilt {
+    /// [TipTilt] `2` x and y items
+    fn n_item(&self) -> usize {
+        2
+    }
+}
+impl OpticalMetrics for SegmentTipTilt {
+    /// [SegmentTipTilt] `7x2` x and y items
+    fn n_item(&self) -> usize {
+        14
+    }
+}
+impl OpticalMetrics for SegmentPiston {
+    /// [SegmentPiston] `7` items
+    fn n_item(&self) -> usize {
+        7
+    }
+}
+
+/// Statistics on [OpticalMetrics]
+pub trait Stats {
+    /// Returns the mean values
+    ///
+    /// Optionally, the statistical moment is evaluated on the last `n_sample`
+    fn mean(&self, n_sample: Option<usize>) -> Vec<f64>
+    where
+        Self: Deref<Target = Vec<f64>> + OpticalMetrics,
+    {
+        let n_item = self.n_item();
+        let n_total = self.len() / n_item;
+        assert!(n_total >= n_sample.unwrap_or(n_total), "not enough samples");
+        (0..n_item)
+            .map(|i| {
+                self.iter()
+                    .skip(i)
+                    .step_by(n_item)
+                    .skip(n_total - n_sample.unwrap_or(n_total))
+                    .sum::<f64>()
+                    / n_sample.unwrap_or(n_total) as f64
+            })
+            .collect()
+    }
+    /// Returns the mean variance values
+    ///
+    /// Optionally, the statistical moment is evaluated on the last `n_sample`
+    fn var(&self, n_sample: Option<usize>) -> Vec<f64>
+    where
+        Self: Deref<Target = Vec<f64>> + OpticalMetrics,
+    {
+        let n_item = self.n_item();
+        let n_total = self.len() / n_item;
+        assert!(n_total >= n_sample.unwrap_or(n_total), "not enough samples");
+        (0..n_item)
+            .zip(self.mean(n_sample))
+            .map(|(i, m)| {
+                self.iter()
+                    .skip(i)
+                    .step_by(n_item)
+                    .skip(n_total - n_sample.unwrap_or(n_total))
+                    .map(|&x| x - m)
+                    .fold(0f64, |a, x| a + x * x)
+                    / n_sample.unwrap_or(n_total) as f64
+            })
+            .collect()
+    }
+    /// Returns the standard deviation values
+    ///
+    /// Optionally, the statistical moment is evaluated on the last `n_sample`
+    fn std(&self, n_sample: Option<usize>) -> Vec<f64>
+    where
+        Self: Deref<Target = Vec<f64>> + OpticalMetrics,
+    {
+        self.var(n_sample).iter().map(|x| x.sqrt()).collect()
+    }
+}
+impl Stats for TipTilt {}
+impl Stats for SegmentTipTilt {}
+impl Stats for SegmentPiston {}
+
 /// Linear Optical Model
 pub struct LOM {
     sens: Vec<OpticalSensitivities>,
@@ -221,22 +340,29 @@ impl LOM {
     /// Returns the pupil average tip and tilt in `[mas]`
     ///
     /// The tip-tilt vector is given as `[x1,y1,...,xi,yi,...,xn,yn]` where i is the time index
-    pub fn tiptilt(&self) -> Vec<f64> {
-        self.sens.as_slice()[OpticalSensitivities::TipTilt(vec![])].into_optics(self.rbm.data())
+    pub fn tiptilt(&self) -> TipTilt {
+        TipTilt(
+            self.sens.as_slice()[OpticalSensitivities::TipTilt(vec![])]
+                .into_optics(self.rbm.data()),
+        )
     }
     /// Returns the segment piston in the telescope exit pupil in `[nm]`
     ///
     /// The segment piston vector is given as `[p11,p21,...,p71,...,p1i,p2i,...,p7i,...,p1n,p2n,...,p7n]` where i is the time index
-    pub fn segment_piston(&self) -> Vec<f64> {
-        self.sens.as_slice()[OpticalSensitivities::SegmentPiston(vec![])]
-            .into_optics(self.rbm.data())
+    pub fn segment_piston(&self) -> SegmentPiston {
+        SegmentPiston(
+            self.sens.as_slice()[OpticalSensitivities::SegmentPiston(vec![])]
+                .into_optics(self.rbm.data()),
+        )
     }
     /// Returns the segment averaged tip and tilt in the telescope exit pupil in `[mas]`
     ///
     /// The segment tip-tilt vector is given as `[x11,x21,...,x71,y11,y21,...,y71,...,x1i,x2i,...,x7i,y1i,y2i,...,y7i,...,x1n,x2n,...,x7n,y1n,y2n,...,y7n]` where i is the time index
-    pub fn segment_tiptilt(&self) -> Vec<f64> {
-        self.sens.as_slice()[OpticalSensitivities::SegmentTipTilt(vec![])]
-            .into_optics(self.rbm.data())
+    pub fn segment_tiptilt(&self) -> SegmentTipTilt {
+        SegmentTipTilt(
+            self.sens.as_slice()[OpticalSensitivities::SegmentTipTilt(vec![])]
+                .into_optics(self.rbm.data()),
+        )
     }
 }
 
