@@ -3,7 +3,7 @@ use std::iter::FromIterator;
 /// GMT M1 and M2 segment rigid body motions
 ///
 /// The rigid body motions are saved in a matrix with 84 rows and as many columns as the number of time steps
-/// A row has the following format: `\[M2,M1\]` where `\[Mi\]=\[S1,S2,S3,S4,S5,S6,S7\] and `\[Sj\]=\[Tjx,Tjy,Tjz,Rjx,Rjy,Rjz\]`
+/// A row has the following format: `[M2,M1]` where `[Mi]=[S1,S2,S3,S4,S5,S6,S7]` and `[Sj]=[Tjx,Tjy,Tjz,Rjx,Rjy,Rjz]`
 pub struct RigidBodyMotions {
     // sampling frequency
     sampling_frequency: Option<f64>,
@@ -86,10 +86,22 @@ impl RigidBodyMotions {
             (0..self.data.ncols()).map(|i| tau * i as f64).collect()
         }
     }
+    /// Returns the number of rigidbody motions sample `n`
+    pub fn len(&self) -> usize {
+        self.data.ncols()
+    }
+
+    pub fn is_empty(&self) -> bool {
+        self.len() == 0
+    }
     /// Returns a reference to the rigid body motion `[84,n]` matrix
     pub fn data(&self) -> &nalgebra::DMatrix<f64> {
         &self.data
     }
+    /// Consumes the object and returns the rigid body motion `[84,n]` matrix
+    pub fn into_data(self) -> nalgebra::DMatrix<f64> {
+        self.data
+    }
 }
 
 #[cfg(feature = "apache")]
@@ -107,7 +119,10 @@ pub mod parquet {
 
     impl RigidBodyMotions {
         /// Creates a [RigidBodyMotions] from M1 and M2 rigid body motions saved in a [parquet](https://docs.rs/parquet) file
-        pub fn from_parquet<P: AsRef<Path>>(path: P) -> Result<Self> {
+        pub fn from_parquet<P>(path: P) -> Result<Self>
+        where
+            P: AsRef<Path>,
+        {
             let file = File::open(path).unwrap();
             let file_reader = SerializedFileReader::new(file).unwrap();
             let mut arrow_reader = ParquetFileArrowReader::new(Arc::new(file_reader));
